require 'sinatra'
require 'oauth2'
require 'json'
require 'rest-client'

enable :sessions

ENV['client'] = '54JVFIJXUOSQ8U4TT657G1XZ5VJQV7XODGPV12MRA16XBBDA'
ENV['secret'] = '2E47Q7MKEIIKJQ9DGAE2DY5TQS6TLC7CWU88GEK3EOP2SPG4'
ENV['api_server'] = 'https://gears.opswat.com'

unless GEARS_API_CLIENT = ENV['client']
  raise "You must specify the GEARS App Client env variable"
end

unless GEARS_API_SECRET = ENV['secret']
  raise "You must specify the GEARS App Secret env variable"
end

unless GEARS_API_SERVER = ENV['api_server']
  raise "You must specify the API Server env variable"
end

def client
  client ||= OAuth2::Client.new(GEARS_API_CLIENT, GEARS_API_SECRET, {
                :site => GEARS_API_SERVER,
                :authorize_url => '/o/oauth/authorize',
                :token_url => '/o/oauth/token'
              })
end

error do
  erb :error
end

get '/' do
  if session[:access_token]
     redirect '/account'
  else
     erb :index
  end
end

#
# OAuth 2.0 Handlers
#
get '/auth' do
  redirect client.auth_code.authorize_url(:redirect_uri => redirect_uri)
end

get '/oauth2callback' do
  access_token = client.auth_code.get_token(params[:code], :redirect_uri => redirect_uri)
  session[:access_token] = access_token.token
  redirect '/account'
end

#
# API Method: GET /account
# Docs: https://onlinehelp.opswat.com/metaaccess/Account_Details.html
#
get '/account' do
  check_token
  @account = get_response('account')
  erb :accounts
end

#
# API Method: POST /devices
# Docs: https://onlinehelp.opswat.com/metaaccess/Get_Devices.html
#
get '/devices' do
  check_token
  @devices = get_response('devices')
  erb :devices
end

#
# API Method: GET /account/policy
# Docs: https://onlinehelp.opswat.com/metaaccess/Account_Policy.html
#
get '/configuration' do
  check_token
  @configuration = get_response('account/policy')
  erb :configuration
end

#
# API Method: POST /logs
# Docs: https://onlinehelp.opswat.com/metaaccess/Logs.html
#
get '/logs' do
  check_token
  @logs = get_response('logs')
  erb :logs
end

#
# API Method: POST devices/:hwid
# Docs: https://onlinehelp.opswat.com/metaaccess/Get_Devices.html
#
get '/show_device/:device_id' do
  check_token
  @device = get_response('devices/' + params[:device_id])
  erb :device
end

#
# API Method: POST /devices/:value/policy_check
# Docs: https://onlinehelp.opswat.com/metaaccess/Devices_Policy_Check.html
#
get '/show_device/:device_id/policy_check' do
  check_token
  @check = get_response('devices/' + params[:device_id] + '/policy_check')
  erb :check
end

def get_response(url)
  access_token = OAuth2::AccessToken.new(client, session[:access_token])
  JSON.parse(access_token.get(GEARS_API_SERVER + "/o/api/v2/#{url}").body)
end

def check_token
   if !session[:access_token]
      redirect '/'
   end
end

get '/logout' do
  session.clear
  redirect '/'
end

def redirect_uri
  uri = URI.parse(request.url)
  uri.path = '/oauth2callback'
  uri.query = nil
  uri.to_s
end
